/*
   block.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

static int blockW, blockH, windowW, windowH;
static int maxsize,xsize,ysize,xof,yof;

static void blockIn()	/* Random block */
{
    Pixmap      pic;
    XGCValues	gcv;
    GC		gc;
    int		i,x,y,*tmp,tmpindex;

    gcv.function=GXcopy;
    gc=XCreateGC(theDisp,theWindow,GCFunction,&gcv );
    if((pic=XCreatePixmap(theDisp,theWindow,blockW,blockH,
		DefaultDepth(theDisp,theScreenNumber)))==(Pixmap)NULL){
        fprintf(stderr,"Can't create Pixmap. Abort.\n");
	goodbyekiss();
    }
    XPutImage(theDisp,pic,gc,theImage,0,0,0,0,blockW,blockH);

    xof=(windowW-blockW)/2;
    yof=(windowH-blockH)/2;
    xsize=(blockW/app_data.blockSteps)+(blockW%app_data.blockSteps ? 1: 0);
    ysize=(blockH/app_data.blockSteps)+(blockH%app_data.blockSteps ? 1: 0);
    maxsize=xsize*ysize;

    if((tmp=(int *)XtMalloc(sizeof(int) * maxsize))==NULL){
	fprintf(stderr,"Can't allocate memory for tmp index !!\n");
	goodbyekiss();
    }

    for(i=0;i<maxsize;i++) tmp[i]= -1;
    for(i=0;i<maxsize;i++){
	while(1){
	    tmpindex=random() % maxsize;
	    if(tmp[tmpindex]==-1) {
		tmp[tmpindex]=i;
		break;
	    }
	}
	x=(tmpindex % xsize)*app_data.blockSteps;y=(tmpindex / xsize)*app_data.blockSteps;
	if(((x+xof)<0 && (y+yof)<0) 
		|| ((x+xof)>=windowW && (y+yof)>=windowH))
	    continue;
        XCopyArea(theDisp,pic,theWindow,gc,x,y,
		((x+app_data.blockSteps)<blockW ? app_data.blockSteps:blockW-x),
		((y+app_data.blockSteps)<blockH ? app_data.blockSteps:blockH-y),
		x+xof,y+yof);
	myevent();
    }
    XtFree((char *)tmp);
    XFreeGC(theDisp,gc);
    XFreePixmap(theDisp,pic);
}

static void blockOut()	/* Random block */
{
    int		i,x,y,*tmp,tmpindex;

    if((tmp=(int *)XtMalloc(sizeof(int) * maxsize))==NULL){
	fprintf(stderr,"Can't allocate memory for tmp index !!\n");
	goodbyekiss();
    }

    for(i=0;i<maxsize;i++) tmp[i]= -1;
    for(i=0;i<maxsize;i++){
	while(1){
	    tmpindex=random() % maxsize;
	    if(tmp[tmpindex]==-1) {
		tmp[tmpindex]=i;
		break;
	    }
	}
	x=(tmpindex%xsize)*app_data.blockSteps;y=(tmpindex/xsize)*app_data.blockSteps;
	if(((x+xof)<0 && (y+yof)<0) 
		|| ((x+xof)>=windowW && (y+yof)>=windowH))
	    continue;
        XClearArea(theDisp,theWindow,x+xof,y+yof,
		((x+app_data.blockSteps)<blockW ? app_data.blockSteps:blockW-x),
		((y+app_data.blockSteps)<blockH ? app_data.blockSteps:blockH-y),False);
        myevent();
    }
    XtFree((char *)tmp);
}

void xblockshow(char *fname)
{
    blockW = gim.width;
    blockH = gim.height;
    windowW = windowWidth;
    windowH = windowHeight;
    PreDisplay();
    StoreColors(xcolors,gim.mapsize);
    if(app_data.showFileName) createFontWindow(fname);
    blockIn();
    if(app_data.showFileName) raiseFontWindow();
    mysleep(app_data.intervalTime);
}

void postxblockshow(char *fname)
{
    blockOut();
    if(app_data.showFileName) removeFontWindow();
}

