/*
   coloranim.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

static int t[256];

static void SetandStoreColors()
{
   int i,j;

   j=gim.mapsize;
   for (i=0;i<j;i++) {
        fcolors[i].red  =xcolors[t[i]].red;
        fcolors[i].green=xcolors[t[i]].green;
        fcolors[i].blue =xcolors[t[i]].blue;
        fcolors[i].flags=xcolors[i].flags;
        fcolors[i].pixel=xcolors[i].pixel;
    }
    StoreColors(fcolors,gim.mapsize);
    myusleep(1000000/app_data.colorAnimTicks);
}

static void RandomizeIndex()
{
    int i,j,m;

    m=gim.mapsize;
    for(i=0;i<m;i++) t[i]= -1;
    for(i=0;i<m;i++) while(1){
        j=random() % m;
        if(t[j]== -1){
          t[j]=i;break;
        }
    }
}

static void QuickSortColors(int from,int to)
{
    int i,j,mean,tmp;

    i = from;
    j = to;
    mean = (t[i]+t[j])/2;
    while(1){
        while(t[i]<mean) i++;
        while(t[j]>mean) j--;
        if(i<=j){
            tmp=t[i];t[i]=t[j];t[j]=tmp;
            i++;j--;
	    SetandStoreColors();
        }
	else break;
    }
    if(from<j) QuickSortColors(from,j);
    if(i<to) QuickSortColors(i,to);
}

static void ShellSortColors()
{
    int h,i,j,k,m,tmp;

    m=gim.mapsize;
    /* Shell Sort */
    for(h=m>>1;h>0;h>>=1) 
	for(i=m-h,k=1;k<=i;k++) 
		for(j=k-1;(j>=0) && (t[j]>t[j+1]);j-=h){
			tmp=t[j];t[j]=t[j+1];t[j+1]=tmp;
			SetandStoreColors();
		}
}

static void BubbleSortColors()
{
    int i,j,m,tmp;

    m=gim.mapsize;
    /* Bubble Sort */
    for(i=0;i<m-1;i++) for(j=0;j<m-1;j++) if(t[j]>t[j+1]){
        tmp=t[j];t[j]=t[j+1];t[j+1]=tmp;
	SetandStoreColors();
    } 
}

void xcoloranimateshow(char *fname)	/* color animate */
{
    int m;

    PreDisplay();
    if(app_data.showFileName) createFontWindow(fname);
    ShowImage((windowWidth-gim.width)/2,
		(windowHeight-gim.height)/2,
		gim.width,gim.height);
    RandomizeIndex();
    if(strcmp(app_data.colorAnimSort,"bubble")==0)
	BubbleSortColors();
    else if(strcmp(app_data.colorAnimSort,"shell")==0)
	ShellSortColors();
    else /* if(strcmp(app_data.colorAnimSort,"quick")==0) */ {
	m=gim.mapsize;
	QuickSortColors(0,m-1);
    }
    if(app_data.showFileName) raiseFontWindow();
    mysleep(app_data.intervalTime);
}

void postxcoloranimateshow(char *fname)	/* color animate */
{
    if(fontWindow != (Window)NULL) removeFontWindow();
}
