/*
   image.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

void FreeXImage()
{
    if(theImage){                           /* Created by XCreateImage()        */
      if(theImage->data)                    /* Use XFree. Not use XDestroyImage */
        XtFree((char *)(theImage->data));   /* Free pixmap data                 */
      XFree(theImage);                      /* Free XImage structure            */
      theImage = NULL;
    }
}

int CreateXImage()
{
    int i,j,k,l;
    dword bmin=196605,wmax=0;
    byte *sptr,*dptr;
    Colormap cmap;

    if(defpixnum){
        XFreeColors(theDisp,theCmap,pixels,defpixnum,0L);
	defpixnum=0;	
    }
    if(defcolors) {
        XFreeColors(theDisp,theCmap,&pixels[256-defcolors],defcolors,0L);
    	defcolors=0;
    }
    /*
	Compress the colormap entries if possible.
    */
    compressRGBmap();

    if(app_data.private){
	j=(app_data.color < gim.mapsize) ? app_data.color : gim.mapsize;
	for(i=0;i<j;i++){
		if(!XAllocColorCells(theDisp,theCmap,False,NULL,0,&(pixels[i]),1)) break;
		if(app_data.debug) fprintf(stderr,"%d ",pixels[i]);
	}
	if(app_data.debug) fprintf(stderr,"\n");
    }else{
	/* Guarantee system white pixel and black pixel */
	if(gim.mapsize > 254 && app_data.color > 254) reduceRGBMap(254);
	j=(app_data.color < gim.mapsize) ? app_data.color : gim.mapsize;
	for(i=0;i<256;i++){
		if(!XAllocColorCells(theDisp,theCmap,False,NULL,0,&(pixels[i]),1)) break;
	}
	if(i == 256){
		k=256-j;
		if(app_data.debug) fprintf(stderr,"j=%d, k=%d\n",j,k);
		for(l=0;l<j;l++){
			pixels[l]=pixels[k+l];
			if(app_data.debug) fprintf(stderr,"%d ",pixels[l]);
		}
		if(app_data.debug) fprintf(stderr,"\n");

		cmap = DefaultColormap(theDisp, DefaultScreen(theDisp));
		for(l=0;l<k;l++) xcolors[j+l].pixel=l;
		XQueryColors(theDisp,cmap,&xcolors[j], k);
		for(l=0;l<k;l++){
			pixels[j+l] = xcolors[j+l].pixel;
			if(app_data.debug) fprintf(stderr,"%d ",pixels[j+l]);
		}
		if(app_data.debug) fprintf(stderr,"\n");
		defcolors=k;
		i=j;
	}else{
		XFreeColors(theDisp,theCmap,pixels,i,0L);
		for(i=0;i<j;i++){
			if(!XAllocColorCells(theDisp,theCmap,False,NULL,0,&(pixels[i]),1)) break;
			if(app_data.debug) fprintf(stderr,"%d ",pixels[i]);
		}
		if(app_data.debug) fprintf(stderr,"\n");
	}
    }

    if(i<j || app_data.color<gim.mapsize){
    	if(i<2 || app_data.color<2) return(ERR_IMAGE);
	if(app_data.verbose)fprintf(stderr,"reduce: %d -> %d\n",gim.mapsize,i);
        reduceRGBMap(i);
    }

    defpixnum = i;
    sptr=dptr=gim.image;
    for(j=0;j<gim.height;j++)
	for(i=0;i<gim.width;i++,sptr++,dptr++)
		*dptr=(byte)pixels[*sptr];

    for(i=0;i<gim.mapsize;i++){
        xcolors[i].red  =gim.red[i]*257;
        xcolors[i].green=gim.green[i]*257;
        xcolors[i].blue =gim.blue[i]*257;
        xcolors[i].flags=DoRed | DoGreen | DoBlue;
        xcolors[i].pixel=pixels[i];
	/* Find near White and Black color in pixels */
	if(wmax < (gim.red[i] + gim.green[i] + gim.blue[i])){
		wmax=gim.red[i] + gim.green[i] + gim.blue[i];
		wcol=pixels[i];
	}
	if(bmin > (gim.red[i] + gim.green[i] + gim.blue[i])){
		bmin=gim.red[i] + gim.green[i] + gim.blue[i];
		bcol=pixels[i];
	}
    }
    if(app_data.private){
	bcol = dbcol;
    }
    if(defcolors > 1){
	bcol = dbcol;
    }
    theImage=XCreateImage(theDisp,theVisual,8,ZPixmap,0,(char *)gim.image,gim.width,gim.height,8,gim.width);

    if(theImage) return(NO_ERR);
    else return(ERR_IMAGE);
}

void StoreColors(XColor *theColors, int mapsize)
{
    XStoreColors(theDisp,theCmap,theColors,mapsize);
    XSync(theDisp,False);
}

void SetColors(word color, int mapsize)
{
    int i,j;

    j=mapsize;
    for(i=0;i<j;i++){
        fcolors[i].red=fcolors[i].green=fcolors[i].blue=color;
        fcolors[i].flags=DoRed|DoGreen|DoBlue;
        fcolors[i].pixel=pixels[i];
    }
    StoreColors(fcolors,mapsize);
}

void PreDisplay()
{
    SetColors(0x00000000, gim.mapsize);	/* Blaken the display window */
    if((app_data.root || (app_data.full && app_data.override)) && !app_data.private) 
		XInstallColormap(theDisp,theCmap);
    else XSetWindowColormap(theDisp,theWindow,theCmap);
    XSetWindowBackground(theDisp,theWindow,bcol);
    XClearWindow(theDisp,theWindow);
    if(defcolors) XStoreColors(theDisp,theCmap,&xcolors[gim.mapsize],defcolors);
}

void PostDisplay()
{
    if((app_data.root || (app_data.full && app_data.override)) 
	&& !app_data.private 
	&& theVisual->class == PseudoColor )
		XUninstallColormap(theDisp,theCmap);
}

