/*
   monotone.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

#define RedPercent      0.299
#define GreenPercent    0.587
#define BluePercent     0.114

static XColor monocolors[256];
static XColor xcs[256];
static long dr[256],dg[256],db[256];
static int mapsize,errf;

static void FadeColors(int mode)
{
    int i,j;

    if(mode){
      for(j=255;j>0;j -= app_data.fadeSteps){
        for(i=0;i<mapsize;i++){
            fcolors[i].red  =xcs[i].red   * (word)j / 255;
            fcolors[i].green=xcs[i].green * (word)j / 255;
            fcolors[i].blue =xcs[i].blue  * (word)j / 255;
            fcolors[i].flags=xcs[i].flags;
            fcolors[i].pixel=xcs[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.fadeTicks);
      }
    }else{
      for(j=0;j<255;j += app_data.fadeSteps){
        for(i=0;i<mapsize;i++){
            fcolors[i].red  =xcs[i].red   * (word)j / 255;
            fcolors[i].green=xcs[i].green * (word)j / 255;
            fcolors[i].blue =xcs[i].blue  * (word)j / 255;
            fcolors[i].flags=xcs[i].flags;
            fcolors[i].pixel=xcs[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.fadeTicks);
      }
    }
}


static int CheckTone()
{
    int i,j;

    j=mapsize;
    for(i=0;i<j;i++){
	if(   xcolors[i].red   != xcolors[i].green
	   || xcolors[i].red   != xcolors[i].blue
	   || xcolors[i].green != xcolors[i].blue ) break;
    }
    if(i == j) return(0);
    else return(1);
}

static void MonotoneColors()
{
    int i,j;
    double tone;

    j=mapsize;
    for(i=0;i<j;i++){
	tone=xcolors[i].red * RedPercent 
		+ xcolors[i].green * GreenPercent 
		+ xcolors[i].blue * BluePercent;
        monocolors[i].red  =(dword)tone;
        monocolors[i].green=(dword)tone;
        monocolors[i].blue =(dword)tone;
        monocolors[i].flags=xcolors[i].flags;
        monocolors[i].pixel=xcolors[i].pixel;
	dr[i] = (long)(xcolors[i].red   - monocolors[i].red  );
	dg[i] = (long)(xcolors[i].green - monocolors[i].green);
	db[i] = (long)(xcolors[i].blue  - monocolors[i].blue );
    }
}

static void AnimateMonotone(int mode)
{
    int i,j,k;

    k=mapsize;
    if(mode){
      for(j=255;j>0;j-=app_data.monotoneSteps){
        for(i=0;i<k;i++){
            fcolors[i].red  =(dword)((long)monocolors[i].red   + dr[i] * j / 255);
            fcolors[i].green=(dword)((long)monocolors[i].green + dg[i] * j / 255);
            fcolors[i].blue =(dword)((long)monocolors[i].blue  + db[i] * j / 255);
            fcolors[i].flags=monocolors[i].flags;
            fcolors[i].pixel=monocolors[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.monotoneTicks);
      }
    }else{
      for(j=1;j<256;j+=app_data.monotoneSteps){
        for(i=0;i<k;i++){
            fcolors[i].red  =(dword)((long)monocolors[i].red   + dr[i] * j / 255);
            fcolors[i].green=(dword)((long)monocolors[i].green + dg[i] * j / 255);
            fcolors[i].blue =(dword)((long)monocolors[i].blue  + db[i] * j / 255);
            fcolors[i].flags=monocolors[i].flags;
            fcolors[i].pixel=monocolors[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.monotoneTicks);
      }
    }
}

void xmonotoneshow(char *fname)	/* Animate monotone colors */
{
    int i;

    mapsize=gim.mapsize;
    for(i=0; i<mapsize; i++){
        xcs[i].red   = xcolors[i].red;
        xcs[i].green = xcolors[i].green;
        xcs[i].blue  = xcolors[i].blue;
        xcs[i].flags = xcolors[i].flags;
        xcs[i].pixel = xcolors[i].pixel;
    }

    PreDisplay();
    errf=CheckTone();
    if(errf)
	MonotoneColors();
    if(app_data.showFileName) createFontWindow(fname);
    ShowImage((windowWidth-gim.width)/2,
		(windowHeight-gim.height)/2,
		gim.width,gim.height);
    if(app_data.showFileName) raiseFontWindow();
    FadeColors(False);
    StoreColors(xcolors,mapsize);
    if(errf)
	AnimateMonotone(True);
    mysleep(app_data.intervalTime);
}

void postxmonotoneshow(char *fname)	/* Animate monotone colors */
{
    if(errf)
	AnimateMonotone(False);
    FadeColors(True);
    if(app_data.showFileName) removeFontWindow();
}
