/*
   mosaic.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

static byte 	*subImage;
static XImage	*subXImage;

static void drawSubImage(int px,int py,int sx,int sy,int w, int h)
{
	byte	pix;
	int	i,j;
	pix = gim.image[py*gim.width+px];
	for(j=0;j<h;j++){
		for(i=0;i<w;i++){
			subImage[(sy+j)*gim.width+(sx+i)]=pix;
		}
	}
	
}

static void mosaicImage()		/* MOSAIC */
{
    Pixmap      pic;
    XGCValues   gcv;
    GC          gc;
    int		i,j,x,y,xof,yof,xpos,ypos,xmax,ymax;
    int		b_cx,b_cy,b_w,b_h,pb_w,pb_h; /* block */
    int		k,xx,yy,maxsize,xsize,ysize;
    int		*tmp,tmpindex;

    if(gim.width<=1 || gim.height<=1) return;
    if((subImage = (byte *)XtMalloc(gim.width * gim.height)) == NULL){
        fprintf(stderr,"Can't allocate memory for subImage !!\n");
	goodbyekiss();
    }
    subXImage=XCreateImage(theDisp,theVisual,8,ZPixmap,0,(char *)subImage,gim.width,gim.height,8,gim.width);

    gcv.function=GXcopy;
    gc=XCreateGC(theDisp,theWindow,GCFunction,&gcv);
    if((pic=XCreatePixmap(theDisp,theWindow,gim.width,gim.height,
		DefaultDepth(theDisp,theScreenNumber)))==(Pixmap)NULL){
        fprintf(stderr,"Can't create Pixmap. Abort.\n");
	goodbyekiss();
    }

    xof=(windowWidth-gim.width)/2;
    yof=(windowHeight-gim.height)/2;
    xmax=xof+gim.width;
    ymax=yof+gim.height;
    b_w=gim.width/2;
    b_h=gim.height/2;
    if(b_w==0) b_w=1;
    if(b_h==0) b_h=1;
    if(b_w>gim.width)  b_w=gim.width;
    if(b_h>gim.height) b_h=gim.height;
    pb_w=b_w;
    pb_h=b_h;

    if((tmp=(int *)XtMalloc(sizeof(int) * gim.width*gim.height))==NULL){
	fprintf(stderr,"Can't allocate memory for tmp index !!\n");
	goodbyekiss();
    }

    while(1){
	/* Make mosaic */
	for(y=0,j=0;y<(gim.height+b_h);y+=b_h,j++){
	    b_cy=b_h/2 + b_h*j;
	    if(b_cy>=gim.height) b_cy=gim.height-1;
	    for(x=0,i=0;x<(gim.width+b_w);x+=b_w,i++){
	        xpos=(x<gim.width ? x:gim.width)+xof;
		ypos=(y<gim.height ? y:gim.height)+yof;
	        b_cx=b_w/2 + b_w*i;
	        if(b_cx>=gim.width) b_cx=gim.width-1;
		drawSubImage(b_cx,b_cy,xpos-xof,ypos-yof,
				(xpos+b_w)<xmax ? b_w:xmax-xpos,
				(ypos+b_h)<ymax ? b_h:ymax-ypos);
	    }
	}

	if(b_w==(gim.width/2) && b_h==(gim.height/2)) {
	    XPutImage(theDisp,theWindow,gc,subXImage,0,0,xof,yof,gim.width,gim.height);
	}else{
	    pb_w = (b_w < 8) ? pb_w : b_w * 2;
	    pb_h = (b_h < 8) ? pb_h : b_h * 2;
    	    XPutImage(theDisp,pic,gc,subXImage,0,0,0,0,gim.width,gim.height);
	    xsize=(gim.width/pb_w)+(gim.width%pb_w ? 1: 0);
	    ysize=(gim.height/pb_h)+(gim.height%pb_h ? 1: 0);
	    maxsize=xsize*ysize;

	    for(k=0;k<maxsize;k++) tmp[k]= -1;
	    for(k=0;k<maxsize;k++){
		while(1){
			tmpindex=random() % maxsize;
			if(tmp[tmpindex]==-1) {
				tmp[tmpindex]=k;
				break;
			}
		}
		xx=(tmpindex % xsize)*pb_w;
		yy=(tmpindex / xsize)*pb_h;
		if(((xx+xof)<0 && (yy+yof)<0) 
			|| ((xx+xof)>=windowWidth && (yy+yof)>=windowHeight))
			continue;
		XCopyArea(theDisp,pic,theWindow,gc,xx,yy,
			((xx+pb_w)<gim.width ? pb_w:gim.width-xx),
			((yy+pb_h)<gim.height ? pb_h:gim.height-yy),
			xx+xof,yy+yof);
		if(!(maxsize % xsize)){
			myevent();
		}
	    }
	}
	if(b_w==1 || b_h==1) break;
	b_w=b_w/2;if(b_w<1) b_w=1;
	b_h=b_h/2;if(b_h<1) b_h=1;
    }
    XPutImage(theDisp,theWindow,gc,theImage,0,0,xof,yof,gim.width,gim.height);

    XtFree((char *)tmp);
    XFreeGC(theDisp,gc);
    XFreePixmap(theDisp,pic);

    if (subXImage){
      if(subXImage->data)
        XtFree((char *)(subXImage->data)); 
      XFree(subXImage);
      subXImage=NULL;
    }
}

void xmosaicshow(char *fname)
{
    PreDisplay();
    StoreColors(xcolors,gim.mapsize);
    if(app_data.showFileName) createFontWindow(fname);
    mosaicImage();
    ShowImage((windowWidth-gim.width)/2,(windowHeight-gim.height)/2,gim.width,gim.height);
    if(app_data.showFileName) raiseFontWindow();
    mysleep(app_data.intervalTime);
}

void postxmosaicshow(char *fname)
{
    if(fontWindow != (Window)NULL) removeFontWindow();
}
