/*
   puzzle.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

typedef enum {
	UP=0,
	DOWN,
	LEFT,
	RIGHT,
	DMAX
}DIRS;

typedef struct {
	int	dir[DMAX];
	int	col;
	int	row;
}puzzleData;

typedef struct {
	int	dir;
	int	count;
}historyData;

static  puzzleData current;
static	Window *wd=NULL;
static	Window firstw;
static	int xof,yof,pw,ph,pofx,pofy,steps,puzzleCount;
static  historyData *history=NULL;

/* Exchange the block at dir for current block */
static void moveCurrent(int dir,int count)
{
    int x1,y1,x2,y2,col,row;
    Window tmpw;

    while(count--){
	switch(dir){
		case UP   : col=current.col;   row=current.row-1 ; break;
		case DOWN : col=current.col;   row=current.row+1 ; break;
		case LEFT : col=current.col-1; row=current.row   ; break;
		case RIGHT: col=current.col+1; row=current.row   ; break;
	}

	x1=xof + pofx + current.col * steps;
	y1=yof + pofy + current.row * steps;
	x2=xof + pofx + col * steps;
	y2=yof + pofy + row * steps;

	/* Move the block at dir on the bresenham line */
	XRaiseWindow(theDisp,wd[row * pw + col]);
	move_on_bresenham_line(wd[row * pw + col],x2,y2,x1,y1,5);
	XLowerWindow(theDisp,wd[row * pw + col]);

	/* Move the current block on the bresenham line */
	tmpw = wd[current.row * pw + current.col];
	XMoveWindow(theDisp,tmpw,x2,y2);
	wd[current.row * pw + current.col]=wd[row * pw + col];
	wd[row * pw + col]=tmpw;

	/* Update current block informations */
	current.col = col;
	current.row = row;
	if(row == 0)        current.dir[UP]   =0;
	else                current.dir[UP]   =1;
	if(row == (ph - 1)) current.dir[DOWN] =0;
	else                current.dir[DOWN] =1;
	if(col == 0)        current.dir[LEFT] =0;
	else                current.dir[LEFT] =1;
	if(col == (pw - 1)) current.dir[RIGHT]=0;
	else                current.dir[RIGHT]=1;
    }
}

static int Puzzle()
{
	int i,predir,dir,count,countmax,m;
	Window tmpw;
	XImage *subImage;
	Pixmap pic,subPic;
	XGCValues gcv;
	GC gc;

	if(app_data.puzzleSteps > MIN(gim.width,gim.height)) steps = MIN(gim.width,gim.height);
	else steps = app_data.puzzleSteps;
	if(steps == 0) steps = 1;

	xof = (windowWidth - gim.width) / 2;
	yof = (windowHeight - gim.height) / 2;
	if((windowWidth > gim.width) && (windowHeight > gim.height)){
		pw = gim.width / steps;
		ph = gim.height / steps;
		pofx = (gim.width % steps) / 2;
		pofy = (gim.height % steps) / 2;
	}else if ((windowWidth > gim.width) && (windowHeight <= gim.height)){
		pw = gim.width / steps;
		ph = windowHeight / steps;
		pofx = (gim.width % steps) / 2;
		pofy = (windowHeight % steps + gim.height - windowHeight) / 2;
	}else if ((windowWidth <= gim.width) && (windowHeight > gim.height)){
		pw = windowWidth / steps;
		ph = gim.height / steps;
		pofx = (windowWidth % steps + gim.width - windowWidth) / 2;
		pofy = (gim.height % steps) / 2;
	}else if ((windowWidth <= gim.width) && (windowHeight <= gim.height)){
		pw = windowWidth / steps;
		ph = windowHeight / steps;
		pofx = (windowWidth % steps + gim.width - windowWidth) / 2;
		pofy = (windowHeight % steps + gim.height - windowHeight) / 2;
	}

	/* Can't puzzle 1x?, ?x1, 2x2 blocks */
	if(pw==1 || ph==1 || (pw==2 && ph==2)) return(False); 

	gcv.function=GXcopy;
	if((pic=XCreatePixmap(theDisp,theWindow,gim.width,gim.height,
				DefaultDepth(theDisp,theScreenNumber)))==(Pixmap)NULL){
		fprintf(stderr,"Can't create Pixmap. Abort.\n");
		goodbyekiss();
	}
	gc=XCreateGC(theDisp,pic,GCFunction,&gcv );
	XPutImage(theDisp,pic,gc,theImage,0,0,0,0,gim.width,gim.height);
	XFreeGC(theDisp,gc);

	m = pw * ph;
	puzzleCount = m * 2;
	if(!history) history=(historyData *)XtMalloc(sizeof(historyData) * puzzleCount);

	wd = (Window *)XtMalloc(sizeof(Window) * m);

	/* Separate the picture into small blocks */
	firstw=XCreateSimpleWindow(theDisp,theWindow,xof+pofx,yof+pofy,steps,steps,1,bcol,bcol);
	for(i=0;i<m;i++){
		wd[i]=XCreateSimpleWindow(theDisp,theWindow,
			xof + pofx + (i % pw) * steps,
			yof + pofy + (i / pw) * steps,
			steps, steps,
			1,bcol,bcol);

		subPic=XCreatePixmap(theDisp,wd[i],
				steps,steps,DefaultDepth(theDisp,theScreenNumber));
		subImage = XGetImage(theDisp,pic,
			pofx + (i % pw) * steps,
			pofy + (i / pw) * steps,	
			steps, steps, XAllPlanes(),ZPixmap);
		gc=XCreateGC(theDisp,wd[i],GCFunction,&gcv );
		XPutImage(theDisp,subPic,gc,subImage,0,0,0,0,steps,steps);
		XSetWindowBackgroundPixmap(theDisp,wd[i],subPic);

		XDestroyImage(subImage);
		XFreePixmap(theDisp,subPic);
		XFreeGC(theDisp,gc);
	}
	XFreePixmap(theDisp,pic);
	tmpw=wd[0];wd[0]=firstw;firstw=tmpw;


	/* Puzzle (Move block randomly) */
	current.dir[UP]=0;
	current.dir[DOWN]=1;
	current.dir[LEFT]=0;
	current.dir[RIGHT]=1;
	current.col=0;
	current.row=0;
	predir=dir=UP;
	for(i=0;i<puzzleCount;i++){
		while(1){
			dir=random() % DMAX;
			if(predir!=dir && current.dir[dir]){
				switch(dir){
				case UP   : countmax=current.row;      predir=DOWN ; break;
				case DOWN : countmax=ph-current.row-1; predir=UP   ; break;
				case LEFT : countmax=current.col;      predir=RIGHT; break;
				case RIGHT: countmax=pw-current.col-1; predir=LEFT ; break;
				}
				break;
			}
		}
		count=(random() % countmax)+1;
		moveCurrent(dir,count);
		history[i].dir=dir;
		history[i].count=count;
	}

	/* Display all blocks */
	for(i=0;i<m;i++) XMapWindow(theDisp,wd[i]);

	return(True);
}

static void Resolve()
{
	int i,dir;

	/* The current block was placed on history[puzzleCount-1] */
	for(i=puzzleCount-1;i>=0;i--){
		switch(history[i].dir){
		case UP   : dir=DOWN ; break;
		case DOWN : dir=UP   ; break;
		case LEFT : dir=RIGHT; break;
		case RIGHT: dir=LEFT ; break;
		}
		moveCurrent(dir,history[i].count);
		mysleep(1);
	}
	/* Display the first block, Then finished */
	XMapRaised(theDisp,firstw);
}

static void freeResources()
{
	int i,m;

	m = pw * ph;
	if(pw!=1 && ph!=1 && (pw!=2 || ph!=2)) {
		for(i=0;i<m;i++) XDestroyWindow(theDisp,wd[i]);
		XDestroyWindow(theDisp,firstw);
		XtFree((char *)wd);
	}
}

void xpuzzleshow(char *fname)
{
    int errf;

    PreDisplay();
    StoreColors(xcolors,gim.mapsize);
    errf = Puzzle();
    if(app_data.showFileName) createFontWindow(fname);
    ShowImage((windowWidth-gim.width)/2,
		(windowHeight-gim.height)/2,
		gim.width,gim.height);
    if(app_data.showFileName) raiseFontWindow();
    mysleep(3);
    if(errf) Resolve();
    mysleep(app_data.intervalTime);
}

void postxpuzzleshow(char *fname)
{
    if(wd != NULL) {
	freeResources();
	XClearWindow(theDisp,theWindow);
    }
    if(fontWindow != (Window)NULL) removeFontWindow();
}
