/*
 * Copyright 1989, 1990, 1991 Jim Frost
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  The author makes no representations
 * about the suitability of this software for any purpose.  It is
 * provided "as is" without express or implied warranty.
 *
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include "xslideshow.h"

#define MAX_SQUARE_VALUE 32768
dword squareNumberTable[MAX_SQUARE_VALUE];

void compressRGBmap()
{ 
	GIMAGE tim;
	word  *index;
	byte  *used,*sptr;
	int   i,j,k;

	index=(word *)XtMalloc(sizeof(word) * gim.mapsize);
	used=(byte *)XtCalloc(gim.mapsize,sizeof(byte));

	tim.mapsize = 0;
	sptr= gim.image;
	for(j=0;j<gim.height;j++) for(i=0;i<gim.width;i++,sptr++){
		if(used[*sptr] == 0){
			for(k=0;k<(int)tim.mapsize;k++)
				if(tim.red[k] == gim.red[*sptr] && 
					tim.green[k] == gim.green[*sptr] &&
					tim.blue[k] == gim.blue[*sptr]) break;

			index[*sptr]=k;
			used[*sptr]=1;

			if(k == (int)tim.mapsize) {
				tim.red[k]=gim.red[*sptr];
				tim.green[k]=gim.green[*sptr];
				tim.blue[k]=gim.blue[*sptr];
				tim.mapsize++;
			}
		}
		*sptr=(byte) index[*sptr];
	}

	gim.mapsize=tim.mapsize;
	for(i=0;i<(int)tim.mapsize;i++){
		gim.red[i]=tim.red[i];
		gim.green[i]=tim.green[i];
		gim.blue[i]=tim.blue[i];
	}
	XtFree((char *)index);
	XtFree((char *)used);
}

void initSquareNumberTable()
{ 
	dword a;
	for (a=0;a<MAX_SQUARE_VALUE;a++) squareNumberTable[a]= a * a;
}

static dword colorDistance(word a,word b)
{
	return(squareNumberTable[(word)ABS(gim.red[a],gim.red[b]) >> 1] +
		squareNumberTable[(word)ABS(gim.green[a],gim.green[b]) >> 1] +
		squareNumberTable[(word)ABS(gim.blue[a],gim.blue[b]) >> 1]);
}

static word bestColor(word color,dword *rdist)
{ 
	word qcolor,bcolor=0;
	dword qdist,bdist=0xffffffff;

	for(qcolor=color+1;qcolor < (word)gim.mapsize;qcolor++)
		if((qdist=colorDistance(color,qcolor)) && (qdist < bdist)){
			bdist=qdist;
			bcolor=qcolor;
		}
	*rdist=bdist;
	return(bcolor);
}

void reduceRGBMap(dword n)
{ 
	dword cl_num,lowintensity,highintensity,myintensity;
	word  a,lowextreme,highextreme;
	word  bcolor1,bcolor2,nr,ng,nb;
	dword bdist;
	dword *dists;
	word  *same;
	word  *best;

	best=(word *)XtMalloc(sizeof(word) * gim.mapsize);
	dists=(dword *)XtMalloc(sizeof(dword) * gim.mapsize);
	same=(word *)XtMalloc(sizeof(word) * gim.mapsize);

	lowextreme=highextreme=gim.mapsize-1;
	lowintensity=highintensity=gim.red[lowextreme] + gim.green[lowextreme] + gim.blue[lowextreme];
	for(a=0;a < (word)gim.mapsize-1;a++){
		myintensity=gim.red[a]+gim.green[a]+gim.blue[a];
		if(myintensity < lowintensity){
			lowintensity=myintensity;lowextreme=a;
		}else if(myintensity > highintensity){
			highintensity=myintensity;highextreme=a;
		}
	}

	for(a=0;a < (word)gim.mapsize-1;a++) best[a]=bestColor(a,dists+a);
	for(a=0;a < (word)gim.mapsize;a++) same[a]=a;

	cl_num=gim.mapsize;
	while (cl_num-- > n) {

	bdist=0xffffffff; /* a really big number */
	for(a=0;a < (word)gim.mapsize-1;a++)
		if((same[a] == a) && (dists[a] < bdist)){
			bdist= dists[a];
			bcolor1=a;
		}
	bcolor2= same[best[bcolor1]];

	if((bcolor1 == lowextreme) || (bcolor1 == highextreme)){
		nr=gim.red[bcolor1];
		ng=gim.green[bcolor1];
		nb=gim.blue[bcolor1];
	}else if((bcolor2 == lowextreme) || (bcolor2 == highextreme)){
		nr=gim.red[bcolor2];
		ng=gim.green[bcolor2];
		nb=gim.blue[bcolor2];
	}else{
		nr=((dword)(gim.red[bcolor1]) + (dword)(gim.red[bcolor2])) >> 1;
		ng=((dword)(gim.green[bcolor1]) + (dword)(gim.green[bcolor2])) >> 1;
		nb=((dword)(gim.blue[bcolor1]) + (dword)(gim.blue[bcolor2])) >> 1;
	}

	for(a=0;a < (word)gim.mapsize;a++)
		if((same[a] == bcolor1) || (same[a] == bcolor2)){
			same[a]=bcolor1;
			gim.red[a]=nr;
			gim.green[a]=ng;
			gim.blue[a]=nb;
		}

	for (a=0;a < (word)gim.mapsize-1;a++)
		if((best[a] == bcolor1) || (same[a] == bcolor1) || (same[best[a]] == bcolor1))
			best[a]=bestColor(a,dists+a);
	}
	XtFree((char *)best);
	XtFree((char *)dists);
	XtFree((char *)same);

	compressRGBmap();
}

