/*
   reverse.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

static XColor rvcolors[256];
static XColor xcs[256];
static long dr[256],dg[256],db[256];
static int mapsize;

static void FadeColors(int mode)
{
    int i,j;

    if(mode){
      for(j=255;j>0;j -= app_data.fadeSteps){
        for(i=0;i<mapsize;i++){
            fcolors[i].red  =xcs[i].red   * (word)j / 255;
            fcolors[i].green=xcs[i].green * (word)j / 255;
            fcolors[i].blue =xcs[i].blue  * (word)j / 255;
            fcolors[i].flags=xcs[i].flags;
            fcolors[i].pixel=xcs[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.fadeTicks);
      }
    }else{
      for(j=0;j<255;j += app_data.fadeSteps){
        for(i=0;i<mapsize;i++){
            fcolors[i].red  =xcs[i].red   * (word)j / 255;
            fcolors[i].green=xcs[i].green * (word)j / 255;
            fcolors[i].blue =xcs[i].blue  * (word)j / 255;
            fcolors[i].flags=xcs[i].flags;
            fcolors[i].pixel=xcs[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.fadeTicks);
      }
    }
}

static void ReverseColors()
{
    int i,j;

    j=mapsize;
    for(i=0;i<j;i++){
        rvcolors[i].red  =0xffffffff - xcolors[i].red;
        rvcolors[i].green=0xffffffff - xcolors[i].green;
        rvcolors[i].blue =0xffffffff - xcolors[i].blue;
        rvcolors[i].flags=xcolors[i].flags;
        rvcolors[i].pixel=xcolors[i].pixel;
	dr[i] = (long)(xcolors[i].red   - rvcolors[i].red  );
	dg[i] = (long)(xcolors[i].green - rvcolors[i].green);
	db[i] = (long)(xcolors[i].blue  - rvcolors[i].blue );
    }
}

static void AnimateReverse(int mode)
{
    int i,j,k;

    k=mapsize;
    if(mode){
      for(j=255;j>0;j-=app_data.reverseSteps){
        for(i=0;i<k;i++){
            fcolors[i].red  =(dword)((long)rvcolors[i].red   + dr[i] * j / 255);
            fcolors[i].green=(dword)((long)rvcolors[i].green + dg[i] * j / 255);
            fcolors[i].blue =(dword)((long)rvcolors[i].blue  + db[i] * j / 255);
            fcolors[i].flags=rvcolors[i].flags;
            fcolors[i].pixel=rvcolors[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.reverseTicks);
      }
    }else{
      for(j=1;j<256;j+=app_data.reverseSteps){
        for(i=0;i<k;i++){
            fcolors[i].red  =(dword)((long)rvcolors[i].red   + dr[i] * j / 255);
            fcolors[i].green=(dword)((long)rvcolors[i].green + dg[i] * j / 255);
            fcolors[i].blue =(dword)((long)rvcolors[i].blue  + db[i] * j / 255);
            fcolors[i].flags=rvcolors[i].flags;
            fcolors[i].pixel=rvcolors[i].pixel;
        }
        StoreColors(fcolors,mapsize);
        myusleep(1000000/app_data.reverseTicks);
      }
    }
}

void xreverseshow(char *fname)	/* Animate reverse colors */
{
    int i;

    mapsize=gim.mapsize;
    for(i=0; i<mapsize; i++){
        xcs[i].red   = xcolors[i].red;
        xcs[i].green = xcolors[i].green;
        xcs[i].blue  = xcolors[i].blue;
        xcs[i].flags = xcolors[i].flags;
        xcs[i].pixel = xcolors[i].pixel;
    }

    PreDisplay();
    ReverseColors();
    if(app_data.showFileName) createFontWindow(fname);
    ShowImage((windowWidth-gim.width)/2,
		(windowHeight-gim.height)/2,
		gim.width,gim.height);
    if(app_data.showFileName) raiseFontWindow();
    FadeColors(False);
    StoreColors(xcolors,mapsize);
    AnimateReverse(True);
    mysleep(app_data.intervalTime);
}

void postxreverseshow(char *fname)	/* Animate reverse colors */
{
    AnimateReverse(False);
    FadeColors(True);
    if(app_data.showFileName) removeFontWindow();
}
