/*
   shuffle.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#include "xslideshow.h"

static	Window *wd=NULL;
static	int xof,yof,pw,ph,pofx,pofy,steps,*t=NULL;

static void QuickSort(int from,int to)
{
    int i,j,mean,tmp;
    int x1,y1,x2,y2;

    i = from;
    j = to;
    mean = (t[i]+t[j])/2;
    while(1){
        while(t[i]<mean) i++;
        while(t[j]>mean) j--;

        if(i<=j){
	    x1=xof + pofx + (t[i] % pw) * steps;
	    y1=yof + pofy + (t[i] / pw) * steps;
	    x2=xof + pofx + (t[j] % pw) * steps;
	    y2=yof + pofy + (t[j] / pw) * steps;

	    XRaiseWindow(theDisp,wd[j]);
	    move_on_bresenham_line(wd[j],x2,y2,x1,y1,3);
	    XLowerWindow(theDisp,wd[j]);

	    XRaiseWindow(theDisp,wd[i]);
	    move_on_bresenham_line(wd[i],x1,y1,x2,y2,3);
	    XLowerWindow(theDisp,wd[i]);

            tmp=t[i];t[i]=t[j];t[j]=tmp;
	    myusleep(1000000/app_data.shuffleTicks);
            i++;j--;
        }
        else break;
    }

    if(from<j) QuickSort(from,j);
    if(i<to) QuickSort(i,to);
}

static void ShellSort()
{
    int h,i,j,k,m,tmp;
    int x1,y1,x2,y2;

    m = pw * ph;
    /* Shell Sort */
    for(h=m>>1;h>0;h>>=1) for(i=m-h,k=1;k<=i;k++) for(j=k-1;(j>=0) && (t[j]>t[j+1]);j-=h){
	x1=xof + pofx + (t[j] % pw) * steps;
	y1=yof + pofy + (t[j] / pw) * steps;
	x2=xof + pofx + (t[j+1] % pw) * steps;
	y2=yof + pofy + (t[j+1] / pw) * steps;

	XRaiseWindow(theDisp,wd[j+1]);
	move_on_bresenham_line(wd[j+1],x2,y2,x1,y1,3);
	XLowerWindow(theDisp,wd[j+1]);

	XRaiseWindow(theDisp,wd[j]);
	move_on_bresenham_line(wd[j],x1,y1,x2,y2,3);
	XLowerWindow(theDisp,wd[j]);

        tmp=t[j];t[j]=t[j+1];t[j+1]=tmp;
	myusleep(1000000/app_data.shuffleTicks);
    }
}

static void BubbleSort()
{
    int i,j,m,tmp;
    int x1,y1,x2,y2;

    m = pw * ph;
    /* Bubble Sort */
    for(i=0;i<m-1;i++) for(j=0;j<m-1;j++) if(t[j]>t[j+1]){
	x1=xof + pofx + (t[j] % pw) * steps;
	y1=yof + pofy + (t[j] / pw) * steps;
	x2=xof + pofx + (t[j+1] % pw) * steps;
	y2=yof + pofy + (t[j+1] / pw) * steps;

	XRaiseWindow(theDisp,wd[j+1]);
	move_on_bresenham_line(wd[j+1],x2,y2,x1,y1,3);
	XLowerWindow(theDisp,wd[j+1]);

	XRaiseWindow(theDisp,wd[j]);
	move_on_bresenham_line(wd[j],x1,y1,x2,y2,3);
	XLowerWindow(theDisp,wd[j]);

        tmp=t[j];t[j]=t[j+1];t[j+1]=tmp;
	myusleep(1000000/app_data.shuffleTicks);
    } 
}

static int Shuffle()
{
	int i,j,m;
	XImage *subImage;
	Pixmap pic,subPic;
	XGCValues gcv;
	GC gc;

        if(app_data.shuffleSteps > MIN(gim.width,gim.height)) steps = MIN(gim.width,gim.height);
        else steps = app_data.shuffleSteps;
        if(steps == 0) steps = 1;

	xof = (windowWidth - gim.width) / 2;
	yof = (windowHeight - gim.height) / 2;
	if((windowWidth > gim.width) && (windowHeight > gim.height)){
		pw = gim.width / steps;
		ph = gim.height / steps;
		pofx = (gim.width % steps) / 2;
		pofy = (gim.height % steps) / 2;
	}else if ((windowWidth > gim.width) && (windowHeight <= gim.height)){
		pw = gim.width / steps;
		ph = windowHeight / steps;
		pofx = (gim.width % steps) / 2;
		pofy = (windowHeight % steps + gim.height - windowHeight) / 2;
	}else if ((windowWidth <= gim.width) && (windowHeight > gim.height)){
		pw = windowWidth / steps;
		ph = gim.height / steps;
		pofx = (windowWidth % steps + gim.width - windowWidth) / 2;
		pofy = (gim.height % steps) / 2;
	}else if ((windowWidth <= gim.width) && (windowHeight <= gim.height)){
		pw = windowWidth / steps;
		ph = windowHeight / steps;
		pofx = (windowWidth % steps + gim.width - windowWidth) / 2;
		pofy = (windowHeight % steps + gim.height - windowHeight) / 2;
	}

	m = pw * ph;
	if(m==1) return(False);

	gcv.function=GXcopy;
	if((pic=XCreatePixmap(theDisp,theWindow,gim.width,gim.height,
				DefaultDepth(theDisp,theScreenNumber)))==(Pixmap)NULL){
		fprintf(stderr,"Can't create Pixmap. Abort.\n");
		goodbyekiss();
	}
	gc=XCreateGC(theDisp,pic,GCFunction,&gcv );
	XPutImage(theDisp,pic,gc,theImage,0,0,0,0,gim.width,gim.height);
	XFreeGC(theDisp,gc);

	t = (int *)XtMalloc(sizeof(int) * m);
	for(i=0;i<m;i++) t[i]= -1;
	for(i=0;i<m;i++) while(1){
		j=random() % m;
		if(t[j]== -1){
			t[j]=i;break;
		}
	}

	wd = (Window *)XtMalloc(sizeof(Window) * m);
	for(i=0;i<m;i++){
		wd[i]=XCreateSimpleWindow(theDisp,theWindow,
			xof + pofx + (t[i] % pw) * steps,
			yof + pofy + (t[i] / pw) * steps,
			steps, steps,
			1,bcol,bcol);

		subPic=XCreatePixmap(theDisp,wd[i],
				steps,steps,DefaultDepth(theDisp,theScreenNumber));
		subImage = XGetImage(theDisp,pic,
			pofx + (i % pw) * steps,
			pofy + (i / pw) * steps,	
			steps, steps, XAllPlanes(),ZPixmap);
		gc=XCreateGC(theDisp,wd[i],GCFunction,&gcv );
		XPutImage(theDisp,subPic,gc,subImage,0,0,0,0,steps,steps);
		XSetWindowBackgroundPixmap(theDisp,wd[i],subPic);

		XDestroyImage(subImage);
		XFreePixmap(theDisp,subPic);
		XFreeGC(theDisp,gc);
	}

	for(i=0;i<m;i++) XMapWindow(theDisp,wd[i]);

	XFreePixmap(theDisp,pic);
	return(True);
}

static void freeResources()
{
	int i,m;

	m = pw * ph;
	if(m>1) {
		for(i=0;i<m;i++) XDestroyWindow(theDisp,wd[i]);
		XtFree((char *)t);
		XtFree((char *)wd);
	}
}

void xshuffleshow(char *fname)
{
    int errf,m;

    PreDisplay();
    StoreColors(xcolors,gim.mapsize);
    errf = Shuffle();
    if(app_data.showFileName) createFontWindow(fname);
    ShowImage((windowWidth-gim.width)/2,
		(windowHeight-gim.height)/2,
		gim.width,gim.height);
    if(errf){
	mysleep(3);
	if(strcmp(app_data.shuffleSort,"bubble")==0)
	    BubbleSort();
	else if(strcmp(app_data.shuffleSort,"shell")==0)
	    ShellSort();
	else /* if(strcmp(app_data.shuffleSort,"quick")==0) */{
	    m=pw*ph;
	    QuickSort(0,m-1);
	}
    }
    if(app_data.showFileName) raiseFontWindow();
    mysleep(app_data.intervalTime);
}

void postxshuffleshow(char *fname)
{
    if(wd != NULL && t != NULL) {
	freeResources();
	XClearWindow(theDisp,theWindow);
    }
    if(fontWindow != (Window)NULL) removeFontWindow();
}
