/*
   main.c - displays pictures on an X11 window
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

#define IS_MAIN
#include "xslideshow.h"
#include "resource.h"
#include "version.h"

static int list_count,now_count,loop_count;
static XtAppContext app_context;
static char **filelist;
static int *listidx;

void myevent()
{
	XtInputMask mask;
	XEvent event;
	while((mask=XtAppPending(app_context))) {
		if(app_data.debug) {
			if(mask & XtIMXEvent)         fprintf(stderr,"XtIMXEvent\n");
			if(mask & XtIMTimer)          fprintf(stderr,"XtIMTimer\n");
			if(mask & XtIMAlternateInput) fprintf(stderr,"XtIMAlternateInput\n");
		}
		XtAppNextEvent(app_context,&event);
		XtDispatchEvent(&event);
	}
}

void myusleep(dword useconds)
{
	myevent();
	usleep(useconds);
	myevent();
}

void mysleep(dword seconds)
{
	while(seconds--)
		myusleep(1000000);
}

void goodbyekiss(void)
{
    if(theDisp){
	XSync(theDisp,False);
	PostDisplay();
	XtCloseDisplay(theDisp);
        if(app_data.verbose)fprintf(stderr,"goodbye kiss!\n");
    }
    exit(0);
}

static int sign(int x)
{
	if(x == 0) return(0);
	else if(x > 0) return(1);
	else return(-1);
}

void warp(Window w,int x1,int y1,int x2,int y2, int step)
{
	XMoveWindow(theDisp,w,x2,y2);
}

void move_on_bresenham_line(Window w,int x1,int y1,int x2,int y2, int step)
{
	int  i,j;
	int  x,y,dx,dy,sx,sy;
	int  prex,prey,incli;
	int  e,cons1,cons2;

	prex=x1;prey=y1;

	dx = x2 - x1;
	dy = y2 - y1;
	sx = sign(dx); sy = sign(dy);
	dx = ((dx < 0) ? -(dx):(dx)); 
	dy = ((dy < 0) ? -(dy):(dy)); 

	if ( dy > dx ){
	    incli = 1;
	    cons2 = dx << 1;
	    cons1 = cons2 - (dy << 1);
	    e = (dx << 1) - dy;
	    x = x1;
	    y = y1 + sy;
	    j = dy;
	}else{
	    incli = 0;
	    cons2 = dy << 1;
	    cons1 = cons2 - (dx << 1);
	    e = (dy << 1) - dx;
	    y = y1;
	    x = x1 + sx;
	    j = dx;
	}

	for(i=1;i<=j;i++){
	    if(e>0){
		if ( incli == 1 ) x += sx;
		else y += sy;
		e += cons1;
	    }else{
	    	e += cons2;
	    }

	    if(ABS(x,prex) > step || ABS(y,prey) > step){
		myevent();
	        XMoveWindow(theDisp,w,x,y);
		prex=x;prey=y;
	    }
	    if(incli==1) y += sy;
	    else x += sx;
	}
	XMoveWindow(theDisp,w,x2,y2);
}

void removeWindow(Window *win)
{
	if(*win != (Window)NULL){
		XDestroyWindow(theDisp,*win);
		*win=(Window)NULL;
	}
}

void ShowWindow(Window *win,int x,int y,int w,int h)
{
        Pixmap pic;
        XGCValues gcv;
        GC gc;


        *win=XCreateSimpleWindow(theDisp,theWindow,x,y,w,h,0,bcol,bcol);
        pic=XCreatePixmap(theDisp,*win,w,h,DefaultDepth(theDisp,theScreenNumber));
	gcv.function=GXcopy;
        gc=XCreateGC(theDisp,pic,GCFunction,&gcv );
	XPutImage(theDisp,pic,gc,theImage,0,0,0,0,w,h);
        XSetWindowBackgroundPixmap(theDisp,*win,pic);
    	XClearWindow(theDisp,*win);

        XFreePixmap(theDisp,pic);
        XFreeGC(theDisp,gc);

	XMapWindow(theDisp,*win);
}

void ShowImage(int x,int y,int w,int h)
{
	XGCValues   gcv;
	GC          gc;

	gcv.function=GXcopy;
	gc=XCreateGC(theDisp,theWindow,GCFunction,&gcv);
	XPutImage(theDisp,theWindow,gc,theImage,0,0,x,y,w,h);
	XFreeGC(theDisp,gc);
}

void raiseFontWindow()
{
	XMapRaised(theDisp,fontWindow);
	mysleep(1);
	move_on_bresenham_line(fontWindow,
		(windowWidth-textWidth)/2,windowHeight+textHeight,
		(windowWidth-textWidth)/2,windowHeight-textHeight, 1);
}

void removeFontWindow()
{
	if(fontWindow != (Window)NULL){
		move_on_bresenham_line(fontWindow,
			(windowWidth-textWidth)/2,windowHeight-textHeight,
			(windowWidth-textWidth)/2,windowHeight+textHeight, 1);
		mysleep(1);
		XDestroyWindow(theDisp,fontWindow);
		fontWindow=(Window)NULL;
	}
}

void createFontWindow(char *fname)
{
	Pixmap pic;
	int d_h,f_a,f_d;
	XCharStruct overall;
	XGCValues gcv;
	GC gc;

	if(fontWindow != (Window)NULL){
		removeFontWindow();
	}
	XTextExtents(app_data.font,fname,strlen(fname),&d_h,&f_a,&f_d,&overall);
	textWidth=overall.width;
	textHeight=overall.descent+overall.ascent;
        fontWindow=XCreateSimpleWindow(theDisp,theWindow,
            (windowWidth-textWidth)/2,windowHeight /* -textHeight*/ +textHeight,
             textWidth,textHeight,0,bcol,bcol);
	pic=XCreatePixmap(theDisp,fontWindow,textWidth,textHeight,DefaultDepth(theDisp,theScreenNumber));
	gcv.font=app_data.font->fid;
	gcv.function=GXcopy;
	gcv.foreground=wcol;
	gcv.background=bcol;
	gc=XCreateGC(theDisp,pic,GCFont | GCFunction | GCForeground | GCBackground, &gcv );
	XDrawImageString(theDisp,pic,gc,0,overall.ascent,fname,strlen(fname));
	XSetWindowBackgroundPixmap(theDisp,fontWindow,pic);
	XClearWindow(theDisp,fontWindow);
	XFreePixmap(theDisp,pic);
	XFreeGC(theDisp,gc);
}

void showErrorMessages(int numlines, char *message1, char *message2)
{
	Pixmap pic;
	int d_h,f_a,f_d;
	XCharStruct overall;
	int tw1,tw2,th1,th2;
	XGCValues   gcv;
	GC          gc;

	if(fontWindow != (Window)NULL){
		removeFontWindow();
	}
	if(numlines == 1 && message1 != NULL){
		XTextExtents(app_data.font,message1,strlen(message1),&d_h,&f_a,&f_d,&overall);
		textWidth=overall.width;
		textHeight=overall.descent+overall.ascent;
		fontWindow=XCreateSimpleWindow(theDisp,theWindow,
			(windowWidth-textWidth)/2,(windowHeight-textHeight)/2,
			textWidth,textHeight,0,
			BlackPixel(theDisp,theScreenNumber),
			BlackPixel(theDisp,theScreenNumber));
		pic=XCreatePixmap(theDisp,fontWindow,textWidth,textHeight,
			DefaultDepth(theDisp,theScreenNumber));
		XDrawImageString(theDisp,pic,fontGC,0,overall.ascent,message1,strlen(message1));
	}else if(message1 != NULL && message2 != NULL){
		XTextExtents(app_data.font,message1,strlen(message1),
			&d_h,&f_a,&f_d,&overall);
		tw1=overall.width;
		th1=overall.descent+overall.ascent;
		XTextExtents(app_data.font,message2,strlen(message2),
			&d_h,&f_a,&f_d,&overall);
		tw2=overall.width;
		th2=overall.descent+overall.ascent;
		textWidth=(tw1 > tw2) ? tw1 : tw2;
		textHeight=th1+th2;
		fontWindow=XCreateSimpleWindow(theDisp,theWindow,
			(windowWidth-textWidth)/2,(windowHeight-textHeight)/2,
			textWidth,textHeight,0,
			BlackPixel(theDisp,theScreenNumber),
			BlackPixel(theDisp,theScreenNumber));
		pic=XCreatePixmap(theDisp,fontWindow,
			textWidth, textHeight,
			DefaultDepth(theDisp,theScreenNumber));

		gcv.function=GXcopy;
		gc=XCreateGC(theDisp,pic,GCFunction,&gcv);
		XSetForeground(theDisp,gc,BlackPixel(theDisp,theScreenNumber));
		XFillRectangle(theDisp,pic,gc,0,0,textWidth,textHeight);
		XFreeGC(theDisp,gc);

		XDrawImageString(theDisp,pic,fontGC,
			(tw1 > tw2) ? 0 : (tw2-tw1)/2,
			overall.ascent,message1,strlen(message1));
		XDrawImageString(theDisp,pic,fontGC,
			(tw2 > tw1) ? 0 : (tw1-tw2)/2,
			th1+overall.ascent,message2,strlen(message2));
	}
	XSetWindowBackgroundPixmap(theDisp,fontWindow,pic);
	XClearWindow(theDisp,fontWindow);
	XFreePixmap(theDisp,pic);
	XMapRaised(theDisp,fontWindow);
}

int checkFileType(char *fname)
{
	char tmpc[12];
        int tmp[12];
        int i;

	if(app_data.verbose)fprintf(stderr,"%s ",fname);
        if((fp=fopen(fname,"r"))==NULL) return(ERR_FILE_RW);
        for(i=0;i<12;i++){
		tmp[i]=0xff & getc(fp);      
		tmpc[i]=(char)tmp[i];
	}
        if(strncmp(tmpc,"GIF",3)==0) fileType=IS_GIF;
        else if(strncmp(tmpc+6,"JFIF",4)==0) fileType=IS_JPG;
	else if((tmp[0]==0xff) &&
		(tmp[1]==0xd8) &&
		(tmp[2]==0xff) &&
		(tmp[3]==0xfe) &&
		(tmp[4]==0x00)) fileType=IS_JPG;

        else fileType=IS_NONE;
        fclose(fp);
	return(NO_ERR);
}

void XSlideshowMainLoop(Widget topWidget)
{
    int errf;

    while(1){
      if(app_data.verbose) fprintf(stderr,"[%d] <%d/%d> %d : ",loop_count+1,now_count+1,list_count,listidx[now_count]+1);
      if((errf=AnimateShow(topWidget,filelist[listidx[now_count]])) != NO_ERR){

	/* Display the ERROR Messages */
	if(fontWindow != (Window)NULL) removeFontWindow();
	if((app_data.root || (app_data.full && app_data.override)) && !app_data.private) XUninstallColormap(theDisp,theCmap);
	else XSetWindowColormap(theDisp,theWindow,defCmap);
    	XSetWindowBackground(theDisp,theWindow,BlackPixel(theDisp,theScreenNumber));
    	XClearWindow(theDisp,theWindow);
	showErrorMessages(2,errstrs[errf],filelist[listidx[now_count]]);
    	mysleep(app_data.intervalTime);
	removeFontWindow();
      }

      /* Set next picture index */
      now_count++;if(now_count>=list_count){now_count=0;loop_count++;}
      myevent();
    }
}

void ResizeWindow(Widget w, caddr_t data, XEvent *event)
{
    if(event->type == ConfigureNotify){
	XGetWindowAttributes (theDisp,theWindow,&xgwa);
	newWindowWidth=xgwa.width;
	newWindowHeight=xgwa.height;
	if(app_data.verbose){
		fprintf(stderr,"ResizeWindow (%d x %d)\n",newWindowWidth,newWindowHeight);
	}
    }
}

void main(int argc, char **argv)
{
    XSetWindowAttributes xswa;
    XGCValues gcv;
    Widget toplevel,topWidget;
    Arg args[5];
    int ac;
    char *fname[1];
    int  list_size=10;
    int  i=0,j,k,errf;
    char *d;
    int size;
    int optc,cmdlinefiles=0;
    char **optv;
    static String byebye = "Ctrl <Key>c: bye()\n<Key>q: bye()\n";
    static XtActionsRec actions[] = { {"bye", (XtActionProc)goodbyekiss} };
    theDisp=0;

    toplevel=XtAppInitialize(&app_context,"XSlideshow",options,XtNumber(options),&argc,argv,defaults,0,0);
    XtAppAddActions(app_context,(XtActionList)actions,XtNumber(actions));
    XtOverrideTranslations(toplevel,XtParseTranslationTable(byebye));
    XtGetApplicationResources(toplevel,&app_data,resources,XtNumber(resources),NULL,0);

    theDisp=XtDisplay(toplevel);
    theScreenNumber=DefaultScreen(theDisp);
    theVisual=DefaultVisual(theDisp,theScreenNumber);
    defCmap=DefaultColormap(theDisp,theScreenNumber);
    defpixnum=0;
    defcolors=0;
    fontWindow=(Window)NULL;

    if(app_data.verbose)fprintf(stderr,"XSlideshow version %s, patchlevel %s\n",version,patchlevel);

    if(app_data.root){
	theWindow=RootWindowOfScreen(XtScreen(toplevel));
	topWidget=toplevel;
    }else{
	if(app_data.full && app_data.override){
		topWidget=XtCreatePopupShell("topWidget",overrideShellWidgetClass,toplevel,NULL,0);
		XtRealizeWidget(topWidget);
		ac=0;
		XtSetArg(args[ac],XtNborderWidth,0);ac++;
		XtSetArg(args[ac],XtNwidth,DisplayWidth(theDisp,theScreenNumber));ac++;
		XtSetArg(args[ac],XtNheight,DisplayHeight(theDisp,theScreenNumber));ac++;
		XtSetValues(topWidget,args,ac);
		XtMapWidget(topWidget);
		XtMoveWidget(topWidget,0,0);
		theWindow=XtWindow(topWidget);
	}else if(app_data.full && !app_data.override){
		topWidget=toplevel;
		XtRealizeWidget(topWidget);
		ac=0;
		XtSetArg(args[ac],XtNwidth,DisplayWidth(theDisp,theScreenNumber));ac++;
		XtSetArg(args[ac],XtNheight,DisplayHeight(theDisp,theScreenNumber));ac++;
		XtSetValues(topWidget,args,ac);
		theWindow=XtWindow(topWidget);
		XMoveWindow(theDisp,theWindow,0,0);
		sleep(1);
	}else{
		topWidget=toplevel;
		XtRealizeWidget(topWidget);
		theWindow=XtWindow(topWidget);
		xswa.backing_store = WhenMapped;
		XChangeWindowAttributes(theDisp, theWindow, CWBackingStore, &xswa);
		if(!app_data.resize) 
			XtAddEventHandler(topWidget, StructureNotifyMask, FALSE, 
					(XtEventHandler)ResizeWindow, (XtPointer)NULL);
	}
    }

    XGetWindowAttributes (theDisp,theWindow,&xgwa);
    newWindowWidth=windowWidth=xgwa.width;
    newWindowHeight=windowHeight=xgwa.height;
    dwcol=WhitePixel(theDisp,theScreenNumber);
    dbcol=BlackPixel(theDisp,theScreenNumber);
    if(app_data.verbose)fprintf(stderr,"black=%d, white=%d\n",dbcol,dwcol);

    fontGC= DefaultGC(theDisp,theScreenNumber);
    gcv.font=app_data.font->fid;
    gcv.foreground=WhitePixel(theDisp,theScreenNumber);
    gcv.background=BlackPixel(theDisp,theScreenNumber);
    XChangeGC(theDisp,fontGC,GCFont | GCForeground | GCBackground,&gcv);

    if( theVisual->class != PseudoColor ) {	/* Xslideshow works only PseudoColor Display */
      showErrorMessages(1,errstrs[ERR_SCREEN],NULL);
      XtAppMainLoop(app_context);
    }

    if(!app_data.private) theCmap=XCreateColormap(theDisp,theWindow,theVisual,AllocNone);
    else theCmap=DefaultColormap(theDisp,theScreenNumber);
    theImage =NULL;


    optc=argc;
    optv=argv;
    while(--optc){
	++optv;
	if(!strncmp(*optv,"-color",6)) {
		--optc; ++optv; continue;
	}
	else if(strncmp(*optv,"-",1)) cmdlinefiles++;
    }
    if(cmdlinefiles){
	if((filelist=(char **)XtMalloc(sizeof(char *) * cmdlinefiles))==NULL){
		fprintf(stderr,"Can't allocate memory for the file list !!\n");
		exit(-1);
	}
	list_count=0;
	optc=argc;
	optv=argv;
	while(--optc){
		++optv;
		if(!strncmp(*optv,"-color",6)) {
			--optc; ++optv; continue;
		}
		else if(strncmp(*optv,"-",1)) filelist[list_count++]= *optv;
	}
    }else{	/* read file name from application resource */
	fname[0]=app_data.fileList;
	if((filelist=(char **)XtMalloc(sizeof(char *) * list_size))==NULL){
		fprintf(stderr,"Can't allocate memory for the file list !!\n");
		exit(-1);
	}
	filelist[0]=(char *)NULL;
	list_count=0;
	i=0;
	d=fname[0];
	size=strlen(d);
	while (i < size) {
          int end,start=i;
          if (d[i]==' ' || d[i]=='\t' || d[i]=='\n' || d[i]==0) {
              i++;
              continue;
          }
          if (list_size <= list_count){
	    list_size=list_size * 2;
            filelist=(char **)XtRealloc((char *)filelist,sizeof (char *) * list_size);
	  }
          filelist[list_count++]=d + i;
          while (d[i] != 0 && d[i] != '\n') i++;
          end=i;
          while(i > start && (d[i-1]==' ' || d[i-1]=='\t')) i--;
          d[i]=0;
          i=end + 1;
	}
    }

    if(filelist[0] != (char *)NULL){
	if(app_data.verbose) fprintf(stderr,"%s ",app_data.animate);
	if(app_data.verbose) fprintf(stderr,"%s ",app_data.fileSelectionMode);

	/* Create the show order index */
	if((listidx=(int *) XtMalloc (sizeof (int) * list_count))==NULL){
	    fprintf(stderr,"Can't allocate memory for file list index !!\n");
	    exit(-1);
	}
	if(strcmp(app_data.fileSelectionMode,"random") == 0){	/* random order */
	  srandom((long)time((long *)NULL));
	  for(j=0;j<list_count;j++) listidx[j]= -1;
	  for(j=0;j<list_count;j++){
	      while(1){
	          k=random() % list_count;
	          if(listidx[k]==-1) {
	  	    listidx[k]=j;break;
	  	  }
	      }
	  }
	  if(app_data.verbose && app_data.debug){
		fprintf(stderr,"\nTotal file count = %d ,",list_count);
		fprintf(stderr,"Random show order is below.\n");
		for(j=0;j<list_count;j++) fprintf(stderr,"%d ",listidx[j]);
		fprintf(stderr,"\n");
	  }
	}else{	/* sequent order */
	  for(j=0;j<list_count;j++) listidx[j]=j;
	  if(app_data.verbose && app_data.debug) fprintf(stderr,"\nTotal file count = %d\n",list_count);
	}

	/* If 1st is a JPEG file, it takes a minutes */
	if(app_data.verbose) {
	    fprintf(stderr,"The 1st show file is ");
	    for(j=0;j<list_count;j++) if((errf=checkFileType(filelist[listidx[j]])) == NO_ERR) break;
	    fprintf(stderr,"\n");
	    if(fileType==IS_JPG && app_data.readJPEG)
		showErrorMessages(2,"Reading a JPEG file.","Just a moment.");
	}
	initSquareNumberTable();
	now_count=0; loop_count=0;

	XSlideshowMainLoop(topWidget);

    } else {
	/* No listFile */
	showErrorMessages(1,errstrs[ERR_NO_FILE],NULL);
	XtAppMainLoop(app_context);
    }

}

#undef IS_MAIN
