/*
   xslideshow.h
   Author:   Susumu Shiohara,SEIKO EPSON Corp.  (shiohara@io.tpp.epson.co.jp)

                  Copyright 1993 by Susumu Shiohara
              Copyright 1993 by SEIKO EPSON CORPORATION.

                           All Rights Reserved

*/

/* include files */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include "vroot.h"

#if defined(SYSV)	/* For Solaris2.x or other System V OS */
#include "system_v.h"
#endif

#ifdef __hpux
#define	random	rand
#define	srandom	srand
#endif

#define MIN(A,B) ((A)<(B)?(A):(B))
#define MAX(A,B) ((A)<(B)?(B):(A))
#define ABS(A,B) ((A)<(B)?(B)-(A):(A)-(B))

typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

typedef enum {
	RANDOM=0,
	SEQUENT
} SelectMode;

typedef enum {
	IS_NONE=0,
	IS_GIF,
	IS_JPG
} FileType;

typedef enum {
	NO_ERR=0,
	ERR_FILE_RW,
	ERR_GIF_DATA,
	ERR_JPG_DATA,
	ERR_NO_FILE,
	ERR_SCREEN,
	ERR_IMAGE
} ErrStatus;

#if defined (IS_MAIN)
# define WHERE
static char *errstrs[] = {
	"No errs",		/* Never use */
	"Can't open file",	/* File open error */
	"Corrupt GIF file",	/* GIF data error */
	"Corrupt JPEG file",	/* JPEG data error */
	"No valid files",	/* No files */
	"Sorry, xslidehsow support only 256 color display",	/* This Display is not 256 color */
	"Image can't create"	/* Reduce or XImage error */
};
#else
# define WHERE extern
#endif

WHERE Display           *theDisp;
WHERE int		theScreenNumber;
WHERE Colormap          theCmap;
WHERE Colormap		defCmap;
WHERE Window            theWindow;
WHERE Window            fontWindow;
WHERE GC	        fontGC;
WHERE dword		bcol, wcol;
WHERE dword		dbcol, dwcol;
WHERE Visual            *theVisual;
WHERE byte		*Image;
WHERE XImage            *theImage;
WHERE XWindowAttributes xgwa;
WHERE XColor		xcolors[256];
WHERE XColor		fcolors[256];
WHERE Pixel		pixels[256];
WHERE int		windowWidth,windowHeight;
WHERE int		newWindowWidth,newWindowHeight;
WHERE int		textWidth,textHeight;
WHERE int		movep;
WHERE int		defpixnum;
WHERE int		defcolors;
WHERE int		fileType;
WHERE FILE		*fp;

typedef struct {
        Boolean debug;
        Boolean verbose;
        Boolean full;
        Boolean override;
        Boolean resize;
        Boolean private;
        Boolean root;
        Boolean readJPEG;
        Boolean showFileName;
        XFontStruct *font;
        int     color;                          /* Color num */
        int     intervalTime;                   /* Display time */
        char    *fileSelectionMode;             /* random or sequent */
        char    *animate;                 	/* Animation mode */
        int     fadeSteps, fadeTicks;           /* Animate Fade */
        int     wipeSteps, wipeTicks;           /* Animate Wipe */
        int     moveSteps, moveTicks;           /* Animate Move */
        int     blockSteps;                     /* Animate Block */
        int     reverseSteps, reverseTicks;     /* Animate reverse */
        int     flushSteps, flushTicks;         /* Animate flush */
	int	flushCount;
        int     monotoneSteps, monotoneTicks;   /* Animate monotone */
        char    *colorAnimSort;			/* Animate color */
        int     colorAnimTicks;
        char    *shuffleSort;			/* Animate shuffle */
        int     shuffleSteps,shuffleTicks;
        int     puzzleSteps;			/* Animate puzzle */
        char    *fileList;                      /* Display file list */
}ApplicationData, *ApplicationDataPtr;

WHERE ApplicationData app_data;

typedef struct {
  int  width;		/* image width */
  int  height;		/* image height */
  int  mapsize;		/* used color map size */
  word red[256];	/* red color map */
  word green[256];	/* green color map */
  word blue[256];	/* blue color map */
  byte *image;		/* image data */
} GIMAGE;

WHERE GIMAGE gim;

typedef enum {
	SLIDE=0,
	FADE,
	REVERSE,
	MONOTONE,
	FLUSH,
	WIPE,
	BLOCK,
	MOSAIC,
	COLORANIM,
	COLORANIM2,
	SHUFFLE,
	PUZZLE,
	MOVE
} ShowMode;

void xslideshow(char *);
void xfadeshow(char *);
void xreverseshow(char *);
void xmonotoneshow(char *);
void xflushshow(char *);
void xwipeshow(char *);
void xblockshow(char *);
void xmosaicshow(char *);
void xcoloranimateshow(char *);
void xcoloranimate2show(char *);
void xshuffleshow(char *);
void xpuzzleshow(char *);
void xmoveshow(char *);
void postxslideshow(char *);
void postxfadeshow(char *);
void postxreverseshow(char *);
void postxmonotoneshow(char *);
void postxflushshow(char *);
void postxwipeshow(char *);
void postxblockshow(char *);
void postxmosaicshow(char *);
void postxcoloranimateshow(char *);
void postxcoloranimate2show(char *);
void postxshuffleshow(char *);
void postxpuzzleshow(char *);
void postxmoveshow(char *);

static void (*animateFunc[])() = {
	xslideshow,		/* SLIDE      */
	xfadeshow,		/* FADE       */
	xreverseshow,		/* REVERSE    */
	xmonotoneshow,		/* MONOTONE   */
	xflushshow,		/* FLUSH      */
	xwipeshow,		/* WIPE       */
	xblockshow,		/* BLOCK      */
	xmosaicshow,		/* MOSAIC     */
	xcoloranimateshow,	/* COLORANIM  */
	xcoloranimate2show,	/* COLORANIM2 */
	xshuffleshow,		/* SHUFFLE    */
	xpuzzleshow,		/* PUZZLE     */
	xmoveshow		/* MOVE       */
};

static void (*postAnimateFunc[])() = {
	postxslideshow,		/* SLIDE      */
	postxfadeshow,		/* FADE       */
	postxreverseshow,	/* REVERSE    */
	postxmonotoneshow,	/* MONOTONE   */
	postxflushshow,		/* FLUSH      */
	postxwipeshow,		/* WIPE       */
	postxblockshow,		/* BLOCK      */
	postxmosaicshow,	/* MOSAIC     */
	postxcoloranimateshow,	/* COLORANIM  */
	postxcoloranimate2show,	/* COLORANIM2 */
	postxshuffleshow,	/* SHUFFLE    */
	postxpuzzleshow,	/* PUZZLE     */
	postxmoveshow		/* MOVE       */
};


static int animate_strtoenum()
{
        if     (strcmp(app_data.animate,"slide")     == 0) return(SLIDE);
        else if(strcmp(app_data.animate,"fade")      == 0) return(FADE);
        else if(strcmp(app_data.animate,"reverse")   == 0) return(REVERSE);
        else if(strcmp(app_data.animate,"monotone")  == 0) return(MONOTONE);
        else if(strcmp(app_data.animate,"flush")     == 0) return(FLUSH);
        else if(strcmp(app_data.animate,"wipe")      == 0) return(WIPE);
        else if(strcmp(app_data.animate,"block")     == 0) return(BLOCK);
        else if(strcmp(app_data.animate,"mosaic")    == 0) return(MOSAIC);
        else if(strcmp(app_data.animate,"coloranim") == 0) return(COLORANIM);
        else if(strcmp(app_data.animate,"coloranim2")== 0) return(COLORANIM2);
        else if(strcmp(app_data.animate,"shuffle")   == 0) return(SHUFFLE);
        else if(strcmp(app_data.animate,"puzzle")    == 0) return(PUZZLE);
        else if(strcmp(app_data.animate,"move")      == 0) return(MOVE);
        else                                           return(FADE);
}

